@@ -38,6 +38,9 @@
 
 #include <string.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <ifaddrs.h>
 
 #ifdef LIBXFCE4PANEL_CHECK_VERSION
 #if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
@@ -78,9 +81,6 @@ static void wavelan_set_orientation(XfcePanelPlugin* plugin, GtkOrientation orie
 static void
 wavelan_set_state(t_wavelan *wavelan, gint state)
 {  
-  /* state = 0 -> no link, =-1 -> error */
-  TRACE ("Entered wavelan_set_state, state = %u", state);
-
   GdkColor color;
 
   gchar signal_color_bad[] = "#e00000";
@@ -88,6 +88,9 @@ wavelan_set_state(t_wavelan *wavelan, gint state)
   gchar signal_color_good[] = "#e6ff00";
   gchar signal_color_strong[] = "#06c500";
   
+  /* state = 0 -> no link, =-1 -> error */
+  TRACE ("Entered wavelan_set_state, state = %d", state);
+
   if(state > 100)
     state = 100;
 
@@ -158,7 +161,7 @@ wavelan_timer(gpointer data)
       }
       else {
         /* set error */
-        tip = g_strdup(wi_strerror(result));
+        tip = g_strdup(_(wi_strerror(result)));
         wavelan_set_state(wavelan, -1);
       }
     }
@@ -166,9 +169,11 @@ wavelan_timer(gpointer data)
       wavelan_set_state(wavelan, stats.ws_quality);
 
       if (strlen(stats.ws_netname) > 0)
-        tip = g_strdup_printf("%s: %d%s at %dMb/s", stats.ws_netname, stats.ws_quality, stats.ws_qunit, stats.ws_rate);
+        /* Translators: net_name: quality quality_unit at rate Mb/s*/
+        tip = g_strdup_printf(_("%s: %d%s at %dMb/s"), stats.ws_netname, stats.ws_quality, stats.ws_qunit, stats.ws_rate);
       else
-        tip = g_strdup_printf("%d%s at %dMb/s", stats.ws_quality, stats.ws_qunit, stats.ws_rate);
+        /* Translators: quality quality_unit at rate Mb/s*/
+        tip = g_strdup_printf(_("%d%s at %dMb/s"), stats.ws_quality, stats.ws_qunit, stats.ws_rate);
     }
   }
   else {
@@ -186,16 +191,6 @@ wavelan_timer(gpointer data)
   return(TRUE);
 }
 
-inline guint
-timeout_add_seconds(guint interval, GSourceFunc function, gpointer data)
-{
-#if GLIB_CHECK_VERSION( 2,14,0 )
-  return g_timeout_add_seconds(interval, function, data);
-#else
-  return g_timeout_add(interval*1000, function, data);
-#endif
-}
-
 static void
 wavelan_reset(t_wavelan *wavelan)
 {
@@ -216,7 +211,7 @@ wavelan_reset(t_wavelan *wavelan)
     if ((wavelan->device = wi_open(wavelan->interface)) != NULL) {
       /* register the update timer */
       TRACE ("Opened device");
-      wavelan->timer_id = timeout_add_seconds(1, wavelan_timer, wavelan);
+      wavelan->timer_id = g_timeout_add_seconds(1, wavelan_timer, wavelan);
     }
   }
 }
@@ -226,29 +221,26 @@ static GList*
 wavelan_query_interfaces (void)
 {
   GList *interfaces = NULL;
-  gchar  line[1024];
-  FILE  *fp;
-  gint   n;
+  struct ifaddrs *ifaddr, *ifa;
 
   TRACE ("Entered wavelan_query_interface");
-  
-  fp = popen ("/sbin/ifconfig -a", "r");
-  if (fp != NULL)
-    {
-      while (fgets (line, 1024, fp) != NULL)
-        {
-          if (!isalpha (*line))
-            continue;
-
-          for (n = 0; isalnum (line[n]); ++n);
-          line[n] = '\0';
-
-          interfaces = g_list_append (interfaces, g_strdup (line));
-        }
-
-      pclose (fp);
-    }
 
+  if (getifaddrs(&ifaddr) == -1) {
+    return NULL;
+  }
+  for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+    if (ifa->ifa_addr == NULL)
+      continue;
+#if defined(AF_LINK) /* BSD */
+    if (ifa->ifa_addr->sa_family == AF_LINK)
+#elif defined(AF_PACKET) /* linux */
+    if (ifa->ifa_addr->sa_family == AF_PACKET)
+#else
+#error "couldnt find a way to get address family on your system"
+#endif
+      interfaces = g_list_append (interfaces, g_strdup (ifa->ifa_name));
+  }
+  freeifaddrs(ifaddr);
   return interfaces;
 }
 
@@ -309,6 +301,7 @@ wavelan_new(XfcePanelPlugin *plugin)
 
   wavelan->signal_colors = TRUE;
   wavelan->show_icon = TRUE;
+  wavelan->state = -2;
 
   wavelan->plugin = plugin;
   
@@ -535,12 +528,13 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   gtk_widget_show(label);
 
   interfaces = wavelan_query_interfaces ();
-  combo = gtk_combo_new ();
-  gtk_combo_set_popdown_strings (GTK_COMBO (combo), interfaces);
+  combo = gtk_combo_box_text_new_with_entry ();
+  for (lp = interfaces; lp != NULL; lp = lp->next)
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), lp->data);
   gtk_widget_show (combo);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
 
-  interface = GTK_COMBO (combo)->entry;
+  interface = gtk_bin_get_child (GTK_BIN (combo));
   if (wavelan->interface != NULL)
     gtk_entry_set_text(GTK_ENTRY(interface), wavelan->interface);
   g_signal_connect(interface, "changed", G_CALLBACK(wavelan_interface_changed),
@@ -562,7 +556,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
   
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_widget_show(hbox);
-  autohide_missing = gtk_check_button_new_with_label(_("Autohide when no hardware present"));
+  autohide_missing = gtk_check_button_new_with_mnemonic(_("Autohide when no _hardware present"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autohide_missing), 
       wavelan->autohide_missing);
   g_signal_connect(autohide_missing, "toggled", 
@@ -581,7 +575,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_widget_show(hbox);
-  signal_colors = gtk_check_button_new_with_label(_("Enable signal quality colors"));
+  signal_colors = gtk_check_button_new_with_mnemonic(_("Enable sig_nal quality colors"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(signal_colors), 
       wavelan->signal_colors);
   g_signal_connect(signal_colors, "toggled", 
@@ -592,7 +586,7 @@ wavelan_create_options (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 
   hbox = gtk_hbox_new(FALSE, 2);
   gtk_widget_show(hbox);
-  show_icon = gtk_check_button_new_with_label(_("Show icon"));
+  show_icon = gtk_check_button_new_with_mnemonic(_("Show _icon"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_icon), 
       wavelan->show_icon);
   g_signal_connect(show_icon, "toggled", 
@@ -632,11 +626,10 @@ wavelan_show_about (XfcePanelPlugin *plugin, t_wavelan *wavelan)
 static void
 wavelan_construct (XfcePanelPlugin *plugin)
 {
-  
-  TRACE ("Entered wavelan_construct");
-  
   t_wavelan *wavelan = wavelan_new(plugin);
 
+  TRACE ("Entered wavelan_construct");
+
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
   g_signal_connect (plugin, "orientation-changed",
